# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/queue/v1/queue.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class QueueSendRequest(betterproto.Message):
    """Request to push a single event to a queue"""

    queue: str = betterproto.string_field(1)
    """
    The Nitric name for the queue this will automatically be resolved to the
    provider specific queue identifier.
    """

    task: "NitricTask" = betterproto.message_field(2)
    """The task to push to the queue"""


@dataclass(eq=False, repr=False)
class QueueSendResponse(betterproto.Message):
    """Result of pushing a single task to a queue"""

    pass


@dataclass(eq=False, repr=False)
class QueueSendBatchRequest(betterproto.Message):
    queue: str = betterproto.string_field(1)
    """
    The Nitric name for the queue this will automatically be resolved to the
    provider specific queue identifier.
    """

    tasks: List["NitricTask"] = betterproto.message_field(2)
    """Array of tasks to push to the queue"""


@dataclass(eq=False, repr=False)
class QueueSendBatchResponse(betterproto.Message):
    """Response for sending a collection of tasks"""

    failed_tasks: List["FailedTask"] = betterproto.message_field(1)
    """A list of tasks that failed to be queued"""


@dataclass(eq=False, repr=False)
class QueueReceiveRequest(betterproto.Message):
    queue: str = betterproto.string_field(1)
    """
    The nitric name for the queue this will automatically be resolved to the
    provider specific queue identifier.
    """

    depth: int = betterproto.int32_field(2)
    """
    The max number of items to pop off the queue, may be capped by provider
    specific limitations
    """


@dataclass(eq=False, repr=False)
class QueueReceiveResponse(betterproto.Message):
    tasks: List["NitricTask"] = betterproto.message_field(1)
    """Array of tasks popped off the queue"""


@dataclass(eq=False, repr=False)
class QueueCompleteRequest(betterproto.Message):
    queue: str = betterproto.string_field(1)
    """
    The nitric name for the queue  this will automatically be resolved to the
    provider specific queue identifier.
    """

    lease_id: str = betterproto.string_field(2)
    """Lease id of the task to be completed"""


@dataclass(eq=False, repr=False)
class QueueCompleteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class FailedTask(betterproto.Message):
    task: "NitricTask" = betterproto.message_field(1)
    """The task that failed to be pushed"""

    message: str = betterproto.string_field(2)
    """A message describing the failure"""


@dataclass(eq=False, repr=False)
class NitricTask(betterproto.Message):
    """A task to be sent or received from a queue."""

    id: str = betterproto.string_field(1)
    """A unique id for the task"""

    lease_id: str = betterproto.string_field(2)
    """
    The lease id unique to the pop request, this must be used to complete,
    extend the lease or release the task.
    """

    payload_type: str = betterproto.string_field(3)
    """A content hint for the tasks payload"""

    payload: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(4)
    """The payload of the task"""


class QueueServiceStub(betterproto.ServiceStub):
    async def send(
        self,
        queue_send_request: "QueueSendRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueueSendResponse":
        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/Send",
            queue_send_request,
            QueueSendResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def send_batch(
        self,
        queue_send_batch_request: "QueueSendBatchRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueueSendBatchResponse":
        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/SendBatch",
            queue_send_batch_request,
            QueueSendBatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def receive(
        self,
        queue_receive_request: "QueueReceiveRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueueReceiveResponse":
        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/Receive",
            queue_receive_request,
            QueueReceiveResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def complete(
        self,
        queue_complete_request: "QueueCompleteRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueueCompleteResponse":
        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/Complete",
            queue_complete_request,
            QueueCompleteResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueueServiceBase(ServiceBase):
    async def send(self, queue_send_request: "QueueSendRequest") -> "QueueSendResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def send_batch(
        self, queue_send_batch_request: "QueueSendBatchRequest"
    ) -> "QueueSendBatchResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def receive(
        self, queue_receive_request: "QueueReceiveRequest"
    ) -> "QueueReceiveResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def complete(
        self, queue_complete_request: "QueueCompleteRequest"
    ) -> "QueueCompleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_send(
        self, stream: "grpclib.server.Stream[QueueSendRequest, QueueSendResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.send(request)
        await stream.send_message(response)

    async def __rpc_send_batch(
        self,
        stream: "grpclib.server.Stream[QueueSendBatchRequest, QueueSendBatchResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.send_batch(request)
        await stream.send_message(response)

    async def __rpc_receive(
        self, stream: "grpclib.server.Stream[QueueReceiveRequest, QueueReceiveResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.receive(request)
        await stream.send_message(response)

    async def __rpc_complete(
        self,
        stream: "grpclib.server.Stream[QueueCompleteRequest, QueueCompleteResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.complete(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.queue.v1.QueueService/Send": grpclib.const.Handler(
                self.__rpc_send,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueSendRequest,
                QueueSendResponse,
            ),
            "/nitric.queue.v1.QueueService/SendBatch": grpclib.const.Handler(
                self.__rpc_send_batch,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueSendBatchRequest,
                QueueSendBatchResponse,
            ),
            "/nitric.queue.v1.QueueService/Receive": grpclib.const.Handler(
                self.__rpc_receive,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueReceiveRequest,
                QueueReceiveResponse,
            ),
            "/nitric.queue.v1.QueueService/Complete": grpclib.const.Handler(
                self.__rpc_complete,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueCompleteRequest,
                QueueCompleteResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/error/v1/error.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto


@dataclass(eq=False, repr=False)
class ErrorScope(betterproto.Message):
    service: str = betterproto.string_field(1)
    """The API service invoked, e.g. 'Service.Method'."""

    plugin: str = betterproto.string_field(2)
    """The plugin method invoked, e.g. 'PluginService.Method'."""

    args: Dict[str, str] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """
    The plugin method arguments, ensure only non-sensitive data is specified.
    """


@dataclass(eq=False, repr=False)
class ErrorDetails(betterproto.Message):
    message: str = betterproto.string_field(1)
    """
    The developer error message, explaining the error and ideally solution.
    """

    cause: str = betterproto.string_field(2)
    """The error root cause."""

    scope: "ErrorScope" = betterproto.message_field(3)
    """The scope of the error."""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/faas/v1/faas.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterable,
    AsyncIterator,
    Dict,
    Iterable,
    List,
    Optional,
    Union,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class ClientMessage(betterproto.Message):
    """Messages the client is able to send to the server"""

    id: str = betterproto.string_field(1)
    """Client message ID, used to pair requests/responses"""

    init_request: "InitRequest" = betterproto.message_field(2, group="content")
    """
    Client initialisation request A worker will not be eligible for triggers
    until it has identified itself
    """

    trigger_response: "TriggerResponse" = betterproto.message_field(3, group="content")
    """Client responsding with result of a trigger"""


@dataclass(eq=False, repr=False)
class ServerMessage(betterproto.Message):
    """Messages the server is able to send to the client"""

    id: str = betterproto.string_field(1)
    """Server message ID, used to pair requests/responses"""

    init_response: "InitResponse" = betterproto.message_field(2, group="content")
    """
    Server responding with client configuration details to an InitRequest
    """

    trigger_request: "TriggerRequest" = betterproto.message_field(3, group="content")
    """Server requesting client to process a trigger"""


@dataclass(eq=False, repr=False)
class ApiWorkerScopes(betterproto.Message):
    scopes: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ApiWorkerOptions(betterproto.Message):
    security: Dict[str, "ApiWorkerScopes"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Apply security definitions to this operation"""

    security_disabled: bool = betterproto.bool_field(2)
    """
    explicitly disable security for this endpoint We need to do this as the
    default value of a repeated field is always empty so there is no way of
    knowing if security is explicitly disabled
    """


@dataclass(eq=False, repr=False)
class ApiWorker(betterproto.Message):
    api: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)
    methods: List[str] = betterproto.string_field(3)
    options: "ApiWorkerOptions" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SubscriptionWorker(betterproto.Message):
    topic: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ScheduleWorker(betterproto.Message):
    key: str = betterproto.string_field(1)
    rate: "ScheduleRate" = betterproto.message_field(10, group="cadence")
    cron: "ScheduleCron" = betterproto.message_field(11, group="cadence")


@dataclass(eq=False, repr=False)
class ScheduleRate(betterproto.Message):
    rate: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ScheduleCron(betterproto.Message):
    cron: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class InitRequest(betterproto.Message):
    """
    InitRequest - Identifies a worker as ready to recieve triggers This message
    will contain information on the type of triggers that a worker is capable
    of handling
    """

    api: "ApiWorker" = betterproto.message_field(10, group="Worker")
    subscription: "SubscriptionWorker" = betterproto.message_field(11, group="Worker")
    schedule: "ScheduleWorker" = betterproto.message_field(12, group="Worker")


@dataclass(eq=False, repr=False)
class InitResponse(betterproto.Message):
    """Placeholder message"""

    pass


@dataclass(eq=False, repr=False)
class TriggerRequest(betterproto.Message):
    """The server has a trigger for the client to handle"""

    data: bytes = betterproto.bytes_field(1)
    """The data in the trigger"""

    mime_type: str = betterproto.string_field(2)
    """Should we supply a mime type for the data? Or rely on context?"""

    http: "HttpTriggerContext" = betterproto.message_field(3, group="context")
    topic: "TopicTriggerContext" = betterproto.message_field(4, group="context")


@dataclass(eq=False, repr=False)
class HeaderValue(betterproto.Message):
    value: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryValue(betterproto.Message):
    value: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class HttpTriggerContext(betterproto.Message):
    method: str = betterproto.string_field(1)
    """The request method"""

    path: str = betterproto.string_field(2)
    """The path of the request"""

    headers_old: Dict[str, str] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """
    The old request headers (preserving for backwards compatibility) TODO:
    Remove in 1.0
    """

    query_params_old: Dict[str, str] = betterproto.map_field(
        4, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """
    The old query params (preserving for backwards compatibility) TODO: Remove
    in 1.0
    """

    headers: Dict[str, "HeaderValue"] = betterproto.map_field(
        5, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """HTTP request headers"""

    query_params: Dict[str, "QueryValue"] = betterproto.map_field(
        6, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """HTTP Query params"""

    path_params: Dict[str, str] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """HTTP Path parameters"""

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("headers_old"):
            warnings.warn(
                "HttpTriggerContext.headers_old is deprecated", DeprecationWarning
            )
        if self.is_set("query_params_old"):
            warnings.warn(
                "HttpTriggerContext.query_params_old is deprecated", DeprecationWarning
            )


@dataclass(eq=False, repr=False)
class TopicTriggerContext(betterproto.Message):
    topic: str = betterproto.string_field(1)
    """The topic the message was published for"""


@dataclass(eq=False, repr=False)
class TriggerResponse(betterproto.Message):
    """The worker has successfully processed a trigger"""

    data: bytes = betterproto.bytes_field(1)
    """The data returned in the response"""

    http: "HttpResponseContext" = betterproto.message_field(10, group="context")
    """response to a http request"""

    topic: "TopicResponseContext" = betterproto.message_field(11, group="context")
    """response to a topic trigger"""


@dataclass(eq=False, repr=False)
class HttpResponseContext(betterproto.Message):
    """
    Specific HttpResponse message Note this does not have to be handled by the
    User at all but they will have the option of control If they choose...
    """

    headers_old: Dict[str, str] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """Old HTTP response headers (deprecated) TODO: Remove in 1.0"""

    status: int = betterproto.int32_field(2)
    """The HTTP status of the request"""

    headers: Dict[str, "HeaderValue"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """HTTP response headers"""

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("headers_old"):
            warnings.warn(
                "HttpResponseContext.headers_old is deprecated", DeprecationWarning
            )


@dataclass(eq=False, repr=False)
class TopicResponseContext(betterproto.Message):
    """
    Specific event response message We do not accept responses for events only
    whether or not they were successfully processed
    """

    success: bool = betterproto.bool_field(1)
    """Success status of the handled event"""


class FaasServiceStub(betterproto.ServiceStub):
    async def trigger_stream(
        self,
        client_message_iterator: Union[
            AsyncIterable["ClientMessage"], Iterable["ClientMessage"]
        ],
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["ServerMessage"]:
        async for response in self._stream_stream(
            "/nitric.faas.v1.FaasService/TriggerStream",
            client_message_iterator,
            ClientMessage,
            ServerMessage,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response


class FaasServiceBase(ServiceBase):
    async def trigger_stream(
        self, client_message_iterator: AsyncIterator["ClientMessage"]
    ) -> AsyncIterator["ServerMessage"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_trigger_stream(
        self, stream: "grpclib.server.Stream[ClientMessage, ServerMessage]"
    ) -> None:
        request = stream.__aiter__()
        await self._call_rpc_handler_server_stream(
            self.trigger_stream,
            stream,
            request,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.faas.v1.FaasService/TriggerStream": grpclib.const.Handler(
                self.__rpc_trigger_stream,
                grpclib.const.Cardinality.STREAM_STREAM,
                ClientMessage,
                ServerMessage,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/resource/v1/resource.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class ResourceType(betterproto.Enum):
    Api = 0
    Function = 1
    Bucket = 2
    Queue = 3
    Topic = 4
    Schedule = 5
    Subscription = 6
    Collection = 7
    Policy = 8
    Secret = 9


class Action(betterproto.Enum):
    BucketFileList = 0
    """Bucket Permissions: 0XX"""

    BucketFileGet = 1
    BucketFilePut = 2
    BucketFileDelete = 3
    TopicList = 200
    """Topic Permissions: 2XX"""

    TopicDetail = 201
    TopicEventPublish = 202
    QueueSend = 300
    """Queue Permissions: 3XX"""

    QueueReceive = 301
    QueueList = 302
    QueueDetail = 303
    CollectionDocumentRead = 400
    """Collection Permissions: 4XX"""

    CollectionDocumentWrite = 401
    CollectionDocumentDelete = 402
    CollectionQuery = 403
    CollectionList = 404
    SecretPut = 500
    """Secret Permissions: 5XX"""

    SecretAccess = 501


@dataclass(eq=False, repr=False)
class PolicyResource(betterproto.Message):
    principals: List["Resource"] = betterproto.message_field(1)
    actions: List["Action"] = betterproto.enum_field(2)
    resources: List["Resource"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Resource(betterproto.Message):
    type: "ResourceType" = betterproto.enum_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ResourceDeclareRequest(betterproto.Message):
    resource: "Resource" = betterproto.message_field(1)
    policy: "PolicyResource" = betterproto.message_field(10, group="config")
    bucket: "BucketResource" = betterproto.message_field(11, group="config")
    queue: "QueueResource" = betterproto.message_field(12, group="config")
    topic: "TopicResource" = betterproto.message_field(13, group="config")
    collection: "CollectionResource" = betterproto.message_field(14, group="config")
    secret: "SecretResource" = betterproto.message_field(15, group="config")
    api: "ApiResource" = betterproto.message_field(16, group="config")


@dataclass(eq=False, repr=False)
class BucketResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class QueueResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class TopicResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CollectionResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class SecretResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ApiSecurityDefinitionJwt(betterproto.Message):
    """protect your API with JWT authentication"""

    issuer: str = betterproto.string_field(1)
    audiences: List[str] = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ApiSecurityDefinition(betterproto.Message):
    jwt: "ApiSecurityDefinitionJwt" = betterproto.message_field(1, group="definition")


@dataclass(eq=False, repr=False)
class ApiScopes(betterproto.Message):
    scopes: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ApiResource(betterproto.Message):
    security_definitions: Dict[str, "ApiSecurityDefinition"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    Security definitions for the api These may be used by registered routes and
    operations on the API
    """

    security: Dict[str, "ApiScopes"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """root level security for this api"""


@dataclass(eq=False, repr=False)
class ResourceDeclareResponse(betterproto.Message):
    pass


class ResourceServiceStub(betterproto.ServiceStub):
    async def declare(
        self,
        resource_declare_request: "ResourceDeclareRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResourceDeclareResponse":
        return await self._unary_unary(
            "/nitric.resource.v1.ResourceService/Declare",
            resource_declare_request,
            ResourceDeclareResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class ResourceServiceBase(ServiceBase):
    async def declare(
        self, resource_declare_request: "ResourceDeclareRequest"
    ) -> "ResourceDeclareResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_declare(
        self,
        stream: "grpclib.server.Stream[ResourceDeclareRequest, ResourceDeclareResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.declare(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.resource.v1.ResourceService/Declare": grpclib.const.Handler(
                self.__rpc_declare,
                grpclib.const.Cardinality.UNARY_UNARY,
                ResourceDeclareRequest,
                ResourceDeclareResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/storage/v1/storage.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class StoragePreSignUrlRequestOperation(betterproto.Enum):
    """Operation"""

    READ = 0
    WRITE = 1


@dataclass(eq=False, repr=False)
class StorageWriteRequest(betterproto.Message):
    """Request to put (create/update) a storage item"""

    bucket_name: str = betterproto.string_field(1)
    """
    Nitric name of the bucket to store in  this will be automatically resolved
    to the provider specific bucket identifier.
    """

    key: str = betterproto.string_field(2)
    """Key to store the item under"""

    body: bytes = betterproto.bytes_field(3)
    """bytes array to store"""


@dataclass(eq=False, repr=False)
class StorageWriteResponse(betterproto.Message):
    """Result of putting a storage item"""

    pass


@dataclass(eq=False, repr=False)
class StorageReadRequest(betterproto.Message):
    """Request to retrieve a storage item"""

    bucket_name: str = betterproto.string_field(1)
    """
    Nitric name of the bucket to retrieve from  this will be automatically
    resolved to the provider specific bucket identifier.
    """

    key: str = betterproto.string_field(2)
    """Key of item to retrieve"""


@dataclass(eq=False, repr=False)
class StorageReadResponse(betterproto.Message):
    """Returned storage item"""

    body: bytes = betterproto.bytes_field(1)
    """The body bytes of the retrieved storage item"""


@dataclass(eq=False, repr=False)
class StorageDeleteRequest(betterproto.Message):
    """Request to delete a storage item"""

    bucket_name: str = betterproto.string_field(1)
    """Name of the bucket to delete from"""

    key: str = betterproto.string_field(2)
    """Key of item to delete"""


@dataclass(eq=False, repr=False)
class StorageDeleteResponse(betterproto.Message):
    """Result of deleting a storage item"""

    pass


@dataclass(eq=False, repr=False)
class StoragePreSignUrlRequest(betterproto.Message):
    """
    Request to generate a pre-signed URL for a file to perform a specific
    operation, such as read or write.
    """

    bucket_name: str = betterproto.string_field(1)
    """
    Nitric name of the bucket to retrieve from  this will be automatically
    resolved to the provider specific bucket identifier.
    """

    key: str = betterproto.string_field(2)
    """
    Key of item to generate the signed URL for. The URL and the token it
    contains will only be valid for operations on this resource specifically.
    """

    operation: "StoragePreSignUrlRequestOperation" = betterproto.enum_field(3)
    expiry: int = betterproto.uint32_field(4)
    """
    Expiry time in seconds for the token included in the signed URL.  Time
    starts from when the access token is generated, not when this request is
    made.  e.g. time.Now().Add(expiry * time.Second) on the server
    """


@dataclass(eq=False, repr=False)
class StoragePreSignUrlResponse(betterproto.Message):
    url: str = betterproto.string_field(1)
    """
    The pre-signed url, restricted to the operation, resource and expiry time
    specified in the request.
    """


@dataclass(eq=False, repr=False)
class StorageListFilesRequest(betterproto.Message):
    bucket_name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class File(betterproto.Message):
    key: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class StorageListFilesResponse(betterproto.Message):
    files: List["File"] = betterproto.message_field(1)
    """keys of the files in the bucket"""


class StorageServiceStub(betterproto.ServiceStub):
    async def read(
        self,
        storage_read_request: "StorageReadRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "StorageReadResponse":
        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/Read",
            storage_read_request,
            StorageReadResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def write(
        self,
        storage_write_request: "StorageWriteRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "StorageWriteResponse":
        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/Write",
            storage_write_request,
            StorageWriteResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def delete(
        self,
        storage_delete_request: "StorageDeleteRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "StorageDeleteResponse":
        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/Delete",
            storage_delete_request,
            StorageDeleteResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def pre_sign_url(
        self,
        storage_pre_sign_url_request: "StoragePreSignUrlRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "StoragePreSignUrlResponse":
        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/PreSignUrl",
            storage_pre_sign_url_request,
            StoragePreSignUrlResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list_files(
        self,
        storage_list_files_request: "StorageListFilesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "StorageListFilesResponse":
        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/ListFiles",
            storage_list_files_request,
            StorageListFilesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class StorageServiceBase(ServiceBase):
    async def read(
        self, storage_read_request: "StorageReadRequest"
    ) -> "StorageReadResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def write(
        self, storage_write_request: "StorageWriteRequest"
    ) -> "StorageWriteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(
        self, storage_delete_request: "StorageDeleteRequest"
    ) -> "StorageDeleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def pre_sign_url(
        self, storage_pre_sign_url_request: "StoragePreSignUrlRequest"
    ) -> "StoragePreSignUrlResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_files(
        self, storage_list_files_request: "StorageListFilesRequest"
    ) -> "StorageListFilesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_read(
        self, stream: "grpclib.server.Stream[StorageReadRequest, StorageReadResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.read(request)
        await stream.send_message(response)

    async def __rpc_write(
        self, stream: "grpclib.server.Stream[StorageWriteRequest, StorageWriteResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.write(request)
        await stream.send_message(response)

    async def __rpc_delete(
        self,
        stream: "grpclib.server.Stream[StorageDeleteRequest, StorageDeleteResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.delete(request)
        await stream.send_message(response)

    async def __rpc_pre_sign_url(
        self,
        stream: "grpclib.server.Stream[StoragePreSignUrlRequest, StoragePreSignUrlResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.pre_sign_url(request)
        await stream.send_message(response)

    async def __rpc_list_files(
        self,
        stream: "grpclib.server.Stream[StorageListFilesRequest, StorageListFilesResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_files(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.storage.v1.StorageService/Read": grpclib.const.Handler(
                self.__rpc_read,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageReadRequest,
                StorageReadResponse,
            ),
            "/nitric.storage.v1.StorageService/Write": grpclib.const.Handler(
                self.__rpc_write,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageWriteRequest,
                StorageWriteResponse,
            ),
            "/nitric.storage.v1.StorageService/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageDeleteRequest,
                StorageDeleteResponse,
            ),
            "/nitric.storage.v1.StorageService/PreSignUrl": grpclib.const.Handler(
                self.__rpc_pre_sign_url,
                grpclib.const.Cardinality.UNARY_UNARY,
                StoragePreSignUrlRequest,
                StoragePreSignUrlResponse,
            ),
            "/nitric.storage.v1.StorageService/ListFiles": grpclib.const.Handler(
                self.__rpc_list_files,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageListFilesRequest,
                StorageListFilesResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/event/v1/event.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class EventPublishRequest(betterproto.Message):
    """Request to publish an event to a topic"""

    topic: str = betterproto.string_field(1)
    """The name of the topic to publish the event to"""

    event: "NitricEvent" = betterproto.message_field(2)
    """The event to be published"""


@dataclass(eq=False, repr=False)
class EventPublishResponse(betterproto.Message):
    """Result of publishing an event"""

    id: str = betterproto.string_field(1)
    """
    The id of the published message When an id was not supplied one should be
    automatically generated
    """


@dataclass(eq=False, repr=False)
class TopicListRequest(betterproto.Message):
    """Request for the Topic List method"""

    pass


@dataclass(eq=False, repr=False)
class TopicListResponse(betterproto.Message):
    """Topic List Response"""

    topics: List["NitricTopic"] = betterproto.message_field(1)
    """The list of found topics"""


@dataclass(eq=False, repr=False)
class NitricTopic(betterproto.Message):
    """Represents an event topic"""

    name: str = betterproto.string_field(1)
    """The Nitric name for the topic"""


@dataclass(eq=False, repr=False)
class NitricEvent(betterproto.Message):
    """Nitric Event Model"""

    id: str = betterproto.string_field(1)
    """A Unique ID for the Nitric Event"""

    payload_type: str = betterproto.string_field(2)
    """A content hint for the events payload"""

    payload: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(3)
    """The payload of the event"""


class EventServiceStub(betterproto.ServiceStub):
    async def publish(
        self,
        event_publish_request: "EventPublishRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "EventPublishResponse":
        return await self._unary_unary(
            "/nitric.event.v1.EventService/Publish",
            event_publish_request,
            EventPublishResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class TopicServiceStub(betterproto.ServiceStub):
    async def list(
        self,
        topic_list_request: "TopicListRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "TopicListResponse":
        return await self._unary_unary(
            "/nitric.event.v1.TopicService/List",
            topic_list_request,
            TopicListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class EventServiceBase(ServiceBase):
    async def publish(
        self, event_publish_request: "EventPublishRequest"
    ) -> "EventPublishResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_publish(
        self, stream: "grpclib.server.Stream[EventPublishRequest, EventPublishResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.publish(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.event.v1.EventService/Publish": grpclib.const.Handler(
                self.__rpc_publish,
                grpclib.const.Cardinality.UNARY_UNARY,
                EventPublishRequest,
                EventPublishResponse,
            ),
        }


class TopicServiceBase(ServiceBase):
    async def list(self, topic_list_request: "TopicListRequest") -> "TopicListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_list(
        self, stream: "grpclib.server.Stream[TopicListRequest, TopicListResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.list(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.event.v1.TopicService/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                TopicListRequest,
                TopicListResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: validate/validate.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import List

import betterproto


class KnownRegex(betterproto.Enum):
    """WellKnownRegex contain some well-known patterns."""

    UNKNOWN = 0
    HTTP_HEADER_NAME = 1
    """HTTP header name as defined by RFC 7230."""

    HTTP_HEADER_VALUE = 2
    """HTTP header value as defined by RFC 7230."""


@dataclass(eq=False, repr=False)
class FieldRules(betterproto.Message):
    """
    FieldRules encapsulates the rules for each type of field. Depending on the
    field, the correct set should be used to ensure proper validations.
    """

    message: "MessageRules" = betterproto.message_field(17)
    float: "FloatRules" = betterproto.message_field(1, group="type")
    """Scalar Field Types"""

    double: "DoubleRules" = betterproto.message_field(2, group="type")
    int32: "Int32Rules" = betterproto.message_field(3, group="type")
    int64: "Int64Rules" = betterproto.message_field(4, group="type")
    uint32: "UInt32Rules" = betterproto.message_field(5, group="type")
    uint64: "UInt64Rules" = betterproto.message_field(6, group="type")
    sint32: "SInt32Rules" = betterproto.message_field(7, group="type")
    sint64: "SInt64Rules" = betterproto.message_field(8, group="type")
    fixed32: "Fixed32Rules" = betterproto.message_field(9, group="type")
    fixed64: "Fixed64Rules" = betterproto.message_field(10, group="type")
    sfixed32: "SFixed32Rules" = betterproto.message_field(11, group="type")
    sfixed64: "SFixed64Rules" = betterproto.message_field(12, group="type")
    bool: "BoolRules" = betterproto.message_field(13, group="type")
    string: "StringRules" = betterproto.message_field(14, group="type")
    bytes: "BytesRules" = betterproto.message_field(15, group="type")
    enum: "EnumRules" = betterproto.message_field(16, group="type")
    """Complex Field Types"""

    repeated: "RepeatedRules" = betterproto.message_field(18, group="type")
    map: "MapRules" = betterproto.message_field(19, group="type")
    any: "AnyRules" = betterproto.message_field(20, group="type")
    """Well-Known Field Types"""

    duration: "DurationRules" = betterproto.message_field(21, group="type")
    timestamp: "TimestampRules" = betterproto.message_field(22, group="type")


@dataclass(eq=False, repr=False)
class FloatRules(betterproto.Message):
    """FloatRules describes the constraints applied to `float` values"""

    const: float = betterproto.float_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: float = betterproto.float_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: float = betterproto.float_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: float = betterproto.float_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: float = betterproto.float_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[float] = betterproto.float_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[float] = betterproto.float_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class DoubleRules(betterproto.Message):
    """DoubleRules describes the constraints applied to `double` values"""

    const: float = betterproto.double_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: float = betterproto.double_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: float = betterproto.double_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: float = betterproto.double_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: float = betterproto.double_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[float] = betterproto.double_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[float] = betterproto.double_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class Int32Rules(betterproto.Message):
    """Int32Rules describes the constraints applied to `int32` values"""

    const: int = betterproto.int32_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.int32_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.int32_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.int32_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.int32_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.int32_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.int32_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class Int64Rules(betterproto.Message):
    """Int64Rules describes the constraints applied to `int64` values"""

    const: int = betterproto.int64_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.int64_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.int64_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.int64_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.int64_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.int64_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.int64_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class UInt32Rules(betterproto.Message):
    """UInt32Rules describes the constraints applied to `uint32` values"""

    const: int = betterproto.uint32_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.uint32_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.uint32_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.uint32_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.uint32_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.uint32_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.uint32_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class UInt64Rules(betterproto.Message):
    """UInt64Rules describes the constraints applied to `uint64` values"""

    const: int = betterproto.uint64_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.uint64_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.uint64_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.uint64_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.uint64_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.uint64_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.uint64_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class SInt32Rules(betterproto.Message):
    """SInt32Rules describes the constraints applied to `sint32` values"""

    const: int = betterproto.sint32_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.sint32_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.sint32_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.sint32_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.sint32_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.sint32_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.sint32_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class SInt64Rules(betterproto.Message):
    """SInt64Rules describes the constraints applied to `sint64` values"""

    const: int = betterproto.sint64_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.sint64_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.sint64_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.sint64_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.sint64_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.sint64_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.sint64_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class Fixed32Rules(betterproto.Message):
    """Fixed32Rules describes the constraints applied to `fixed32` values"""

    const: int = betterproto.fixed32_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.fixed32_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.fixed32_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.fixed32_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.fixed32_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.fixed32_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.fixed32_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class Fixed64Rules(betterproto.Message):
    """Fixed64Rules describes the constraints applied to `fixed64` values"""

    const: int = betterproto.fixed64_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.fixed64_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.fixed64_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.fixed64_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.fixed64_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.fixed64_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.fixed64_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class SFixed32Rules(betterproto.Message):
    """SFixed32Rules describes the constraints applied to `sfixed32` values"""

    const: int = betterproto.sfixed32_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.sfixed32_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.sfixed32_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.sfixed32_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.sfixed32_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.sfixed32_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.sfixed32_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class SFixed64Rules(betterproto.Message):
    """SFixed64Rules describes the constraints applied to `sfixed64` values"""

    const: int = betterproto.sfixed64_field(1)
    """Const specifies that this field must be exactly the specified value"""

    lt: int = betterproto.sfixed64_field(2)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: int = betterproto.sfixed64_field(3)
    """
    Lte specifies that this field must be less than or equal to the specified
    value, inclusive
    """

    gt: int = betterproto.sfixed64_field(4)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    range is reversed.
    """

    gte: int = betterproto.sfixed64_field(5)
    """
    Gte specifies that this field must be greater than or equal to the
    specified value, inclusive. If the value of Gte is larger than a specified
    Lt or Lte, the range is reversed.
    """

    in_: List[int] = betterproto.sfixed64_field(6)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.sfixed64_field(7)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ignore_empty: bool = betterproto.bool_field(8)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class BoolRules(betterproto.Message):
    """BoolRules describes the constraints applied to `bool` values"""

    const: bool = betterproto.bool_field(1)
    """Const specifies that this field must be exactly the specified value"""


@dataclass(eq=False, repr=False)
class StringRules(betterproto.Message):
    """StringRules describe the constraints applied to `string` values"""

    const: str = betterproto.string_field(1)
    """Const specifies that this field must be exactly the specified value"""

    len: int = betterproto.uint64_field(19)
    """
    Len specifies that this field must be the specified number of characters
    (Unicode code points). Note that the number of characters may differ from
    the number of bytes in the string.
    """

    min_len: int = betterproto.uint64_field(2)
    """
    MinLen specifies that this field must be the specified number of characters
    (Unicode code points) at a minimum. Note that the number of characters may
    differ from the number of bytes in the string.
    """

    max_len: int = betterproto.uint64_field(3)
    """
    MaxLen specifies that this field must be the specified number of characters
    (Unicode code points) at a maximum. Note that the number of characters may
    differ from the number of bytes in the string.
    """

    len_bytes: int = betterproto.uint64_field(20)
    """
    LenBytes specifies that this field must be the specified number of bytes at
    a minimum
    """

    min_bytes: int = betterproto.uint64_field(4)
    """
    MinBytes specifies that this field must be the specified number of bytes at
    a minimum
    """

    max_bytes: int = betterproto.uint64_field(5)
    """
    MaxBytes specifies that this field must be the specified number of bytes at
    a maximum
    """

    pattern: str = betterproto.string_field(6)
    """
    Pattern specifes that this field must match against the specified regular
    expression (RE2 syntax). The included expression should elide any
    delimiters.
    """

    prefix: str = betterproto.string_field(7)
    """
    Prefix specifies that this field must have the specified substring at the
    beginning of the string.
    """

    suffix: str = betterproto.string_field(8)
    """
    Suffix specifies that this field must have the specified substring at the
    end of the string.
    """

    contains: str = betterproto.string_field(9)
    """
    Contains specifies that this field must have the specified substring
    anywhere in the string.
    """

    not_contains: str = betterproto.string_field(23)
    """
    NotContains specifies that this field cannot have the specified substring
    anywhere in the string.
    """

    in_: List[str] = betterproto.string_field(10)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[str] = betterproto.string_field(11)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    email: bool = betterproto.bool_field(12, group="well_known")
    """
    Email specifies that the field must be a valid email address as defined by
    RFC 5322
    """

    hostname: bool = betterproto.bool_field(13, group="well_known")
    """
    Hostname specifies that the field must be a valid hostname as defined by
    RFC 1034. This constraint does not support internationalized domain names
    (IDNs).
    """

    ip: bool = betterproto.bool_field(14, group="well_known")
    """
    Ip specifies that the field must be a valid IP (v4 or v6) address. Valid
    IPv6 addresses should not include surrounding square brackets.
    """

    ipv4: bool = betterproto.bool_field(15, group="well_known")
    """Ipv4 specifies that the field must be a valid IPv4 address."""

    ipv6: bool = betterproto.bool_field(16, group="well_known")
    """
    Ipv6 specifies that the field must be a valid IPv6 address. Valid IPv6
    addresses should not include surrounding square brackets.
    """

    uri: bool = betterproto.bool_field(17, group="well_known")
    """
    Uri specifies that the field must be a valid, absolute URI as defined by
    RFC 3986
    """

    uri_ref: bool = betterproto.bool_field(18, group="well_known")
    """
    UriRef specifies that the field must be a valid URI as defined by RFC 3986
    and may be relative or absolute.
    """

    address: bool = betterproto.bool_field(21, group="well_known")
    """
    Address specifies that the field must be either a valid hostname as defined
    by RFC 1034 (which does not support internationalized domain names or
    IDNs), or it can be a valid IP (v4 or v6).
    """

    uuid: bool = betterproto.bool_field(22, group="well_known")
    """
    Uuid specifies that the field must be a valid UUID as defined by RFC 4122
    """

    well_known_regex: "KnownRegex" = betterproto.enum_field(24, group="well_known")
    """
    WellKnownRegex specifies a common well known pattern defined as a regex.
    """

    strict: bool = betterproto.bool_field(25)
    """
    This applies to regexes HTTP_HEADER_NAME and HTTP_HEADER_VALUE to enable
    strict header validation. By default, this is true, and HTTP header
    validations are RFC-compliant. Setting to false will enable a looser
    validations that only disallows \r\n\0 characters, which can be used to
    bypass header matching rules.
    """

    ignore_empty: bool = betterproto.bool_field(26)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class BytesRules(betterproto.Message):
    """BytesRules describe the constraints applied to `bytes` values"""

    const: bytes = betterproto.bytes_field(1)
    """Const specifies that this field must be exactly the specified value"""

    len: int = betterproto.uint64_field(13)
    """Len specifies that this field must be the specified number of bytes"""

    min_len: int = betterproto.uint64_field(2)
    """
    MinLen specifies that this field must be the specified number of bytes at a
    minimum
    """

    max_len: int = betterproto.uint64_field(3)
    """
    MaxLen specifies that this field must be the specified number of bytes at a
    maximum
    """

    pattern: str = betterproto.string_field(4)
    """
    Pattern specifes that this field must match against the specified regular
    expression (RE2 syntax). The included expression should elide any
    delimiters.
    """

    prefix: bytes = betterproto.bytes_field(5)
    """
    Prefix specifies that this field must have the specified bytes at the
    beginning of the string.
    """

    suffix: bytes = betterproto.bytes_field(6)
    """
    Suffix specifies that this field must have the specified bytes at the end
    of the string.
    """

    contains: bytes = betterproto.bytes_field(7)
    """
    Contains specifies that this field must have the specified bytes anywhere
    in the string.
    """

    in_: List[bytes] = betterproto.bytes_field(8)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[bytes] = betterproto.bytes_field(9)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """

    ip: bool = betterproto.bool_field(10, group="well_known")
    """
    Ip specifies that the field must be a valid IP (v4 or v6) address in byte
    format
    """

    ipv4: bool = betterproto.bool_field(11, group="well_known")
    """
    Ipv4 specifies that the field must be a valid IPv4 address in byte format
    """

    ipv6: bool = betterproto.bool_field(12, group="well_known")
    """
    Ipv6 specifies that the field must be a valid IPv6 address in byte format
    """

    ignore_empty: bool = betterproto.bool_field(14)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class EnumRules(betterproto.Message):
    """EnumRules describe the constraints applied to enum values"""

    const: int = betterproto.int32_field(1)
    """Const specifies that this field must be exactly the specified value"""

    defined_only: bool = betterproto.bool_field(2)
    """
    DefinedOnly specifies that this field must be only one of the defined
    values for this enum, failing on any undefined value.
    """

    in_: List[int] = betterproto.int32_field(3)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[int] = betterproto.int32_field(4)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """


@dataclass(eq=False, repr=False)
class MessageRules(betterproto.Message):
    """
    MessageRules describe the constraints applied to embedded message values.
    For message-type fields, validation is performed recursively.
    """

    skip: bool = betterproto.bool_field(1)
    """
    Skip specifies that the validation rules of this field should not be
    evaluated
    """

    required: bool = betterproto.bool_field(2)
    """Required specifies that this field must be set"""


@dataclass(eq=False, repr=False)
class RepeatedRules(betterproto.Message):
    """RepeatedRules describe the constraints applied to `repeated` values"""

    min_items: int = betterproto.uint64_field(1)
    """
    MinItems specifies that this field must have the specified number of items
    at a minimum
    """

    max_items: int = betterproto.uint64_field(2)
    """
    MaxItems specifies that this field must have the specified number of items
    at a maximum
    """

    unique: bool = betterproto.bool_field(3)
    """
    Unique specifies that all elements in this field must be unique. This
    contraint is only applicable to scalar and enum types (messages are not
    supported).
    """

    items: "FieldRules" = betterproto.message_field(4)
    """
    Items specifies the contraints to be applied to each item in the field.
    Repeated message fields will still execute validation against each item
    unless skip is specified here.
    """

    ignore_empty: bool = betterproto.bool_field(5)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class MapRules(betterproto.Message):
    """MapRules describe the constraints applied to `map` values"""

    min_pairs: int = betterproto.uint64_field(1)
    """
    MinPairs specifies that this field must have the specified number of KVs at
    a minimum
    """

    max_pairs: int = betterproto.uint64_field(2)
    """
    MaxPairs specifies that this field must have the specified number of KVs at
    a maximum
    """

    no_sparse: bool = betterproto.bool_field(3)
    """
    NoSparse specifies values in this field cannot be unset. This only applies
    to map's with message value types.
    """

    keys: "FieldRules" = betterproto.message_field(4)
    """
    Keys specifies the constraints to be applied to each key in the field.
    """

    values: "FieldRules" = betterproto.message_field(5)
    """
    Values specifies the constraints to be applied to the value of each key in
    the field. Message values will still have their validations evaluated
    unless skip is specified here.
    """

    ignore_empty: bool = betterproto.bool_field(6)
    """
    IgnoreEmpty specifies that the validation rules of this field should be
    evaluated only if the field is not empty
    """


@dataclass(eq=False, repr=False)
class AnyRules(betterproto.Message):
    """
    AnyRules describe constraints applied exclusively to the
    `google.protobuf.Any` well-known type
    """

    required: bool = betterproto.bool_field(1)
    """Required specifies that this field must be set"""

    in_: List[str] = betterproto.string_field(2)
    """
    In specifies that this field's `type_url` must be equal to one of the
    specified values.
    """

    not_in: List[str] = betterproto.string_field(3)
    """
    NotIn specifies that this field's `type_url` must not be equal to any of
    the specified values.
    """


@dataclass(eq=False, repr=False)
class DurationRules(betterproto.Message):
    """
    DurationRules describe the constraints applied exclusively to the
    `google.protobuf.Duration` well-known type
    """

    required: bool = betterproto.bool_field(1)
    """Required specifies that this field must be set"""

    const: timedelta = betterproto.message_field(2)
    """Const specifies that this field must be exactly the specified value"""

    lt: timedelta = betterproto.message_field(3)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: timedelta = betterproto.message_field(4)
    """
    Lt specifies that this field must be less than the specified value,
    inclusive
    """

    gt: timedelta = betterproto.message_field(5)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive
    """

    gte: timedelta = betterproto.message_field(6)
    """
    Gte specifies that this field must be greater than the specified value,
    inclusive
    """

    in_: List[timedelta] = betterproto.message_field(7)
    """
    In specifies that this field must be equal to one of the specified values
    """

    not_in: List[timedelta] = betterproto.message_field(8)
    """
    NotIn specifies that this field cannot be equal to one of the specified
    values
    """


@dataclass(eq=False, repr=False)
class TimestampRules(betterproto.Message):
    """
    TimestampRules describe the constraints applied exclusively to the
    `google.protobuf.Timestamp` well-known type
    """

    required: bool = betterproto.bool_field(1)
    """Required specifies that this field must be set"""

    const: datetime = betterproto.message_field(2)
    """Const specifies that this field must be exactly the specified value"""

    lt: datetime = betterproto.message_field(3)
    """
    Lt specifies that this field must be less than the specified value,
    exclusive
    """

    lte: datetime = betterproto.message_field(4)
    """
    Lte specifies that this field must be less than the specified value,
    inclusive
    """

    gt: datetime = betterproto.message_field(5)
    """
    Gt specifies that this field must be greater than the specified value,
    exclusive
    """

    gte: datetime = betterproto.message_field(6)
    """
    Gte specifies that this field must be greater than the specified value,
    inclusive
    """

    lt_now: bool = betterproto.bool_field(7)
    """
    LtNow specifies that this must be less than the current time. LtNow can
    only be used with the Within rule.
    """

    gt_now: bool = betterproto.bool_field(8)
    """
    GtNow specifies that this must be greater than the current time. GtNow can
    only be used with the Within rule.
    """

    within: timedelta = betterproto.message_field(9)
    """
    Within specifies that this field must be within this duration of the
    current time. This constraint can be used alone or with the LtNow and GtNow
    rules.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/secret/v1/secret.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class SecretPutRequest(betterproto.Message):
    """Request to put a secret to a Secret Store"""

    secret: "Secret" = betterproto.message_field(1)
    """The Secret to put to the Secret store"""

    value: bytes = betterproto.bytes_field(2)
    """The value to assign to that secret"""


@dataclass(eq=False, repr=False)
class SecretPutResponse(betterproto.Message):
    """Result from putting the secret to a Secret Store"""

    secret_version: "SecretVersion" = betterproto.message_field(1)
    """The id of the secret"""


@dataclass(eq=False, repr=False)
class SecretAccessRequest(betterproto.Message):
    """Request to get a secret from a Secret Store"""

    secret_version: "SecretVersion" = betterproto.message_field(1)
    """The id of the secret"""


@dataclass(eq=False, repr=False)
class SecretAccessResponse(betterproto.Message):
    """The secret response"""

    secret_version: "SecretVersion" = betterproto.message_field(1)
    """The version of the secret that was requested"""

    value: bytes = betterproto.bytes_field(2)
    """The value of the secret"""


@dataclass(eq=False, repr=False)
class Secret(betterproto.Message):
    """The secret container"""

    name: str = betterproto.string_field(1)
    """The secret name"""


@dataclass(eq=False, repr=False)
class SecretVersion(betterproto.Message):
    """A version of a secret"""

    secret: "Secret" = betterproto.message_field(1)
    """Reference to the secret container"""

    version: str = betterproto.string_field(2)
    """The secret version"""


class SecretServiceStub(betterproto.ServiceStub):
    async def put(
        self,
        secret_put_request: "SecretPutRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SecretPutResponse":
        return await self._unary_unary(
            "/nitric.secret.v1.SecretService/Put",
            secret_put_request,
            SecretPutResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def access(
        self,
        secret_access_request: "SecretAccessRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SecretAccessResponse":
        return await self._unary_unary(
            "/nitric.secret.v1.SecretService/Access",
            secret_access_request,
            SecretAccessResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class SecretServiceBase(ServiceBase):
    async def put(self, secret_put_request: "SecretPutRequest") -> "SecretPutResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def access(
        self, secret_access_request: "SecretAccessRequest"
    ) -> "SecretAccessResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_put(
        self, stream: "grpclib.server.Stream[SecretPutRequest, SecretPutResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.put(request)
        await stream.send_message(response)

    async def __rpc_access(
        self, stream: "grpclib.server.Stream[SecretAccessRequest, SecretAccessResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.access(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.secret.v1.SecretService/Put": grpclib.const.Handler(
                self.__rpc_put,
                grpclib.const.Cardinality.UNARY_UNARY,
                SecretPutRequest,
                SecretPutResponse,
            ),
            "/nitric.secret.v1.SecretService/Access": grpclib.const.Handler(
                self.__rpc_access,
                grpclib.const.Cardinality.UNARY_UNARY,
                SecretAccessRequest,
                SecretAccessResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/document/v1/document.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class Collection(betterproto.Message):
    """Provides a Collection type for storing documents"""

    name: str = betterproto.string_field(1)
    """The collection name"""

    parent: "Key" = betterproto.message_field(2)
    """
    Optional parent key, required when the collection is a sub-collection of
    another document
    """


@dataclass(eq=False, repr=False)
class Key(betterproto.Message):
    """Provides a document identifying key type"""

    collection: "Collection" = betterproto.message_field(1)
    """The item collection"""

    id: str = betterproto.string_field(2)
    """The items unique id"""


@dataclass(eq=False, repr=False)
class Document(betterproto.Message):
    """Provides a return document type"""

    content: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """The document content (JSON object)"""

    key: "Key" = betterproto.message_field(2)
    """The document's unique key, including collection/sub-collections"""


@dataclass(eq=False, repr=False)
class ExpressionValue(betterproto.Message):
    int_value: int = betterproto.int64_field(1, group="kind")
    """Represents an integer value."""

    double_value: float = betterproto.double_field(2, group="kind")
    """Represents a double value."""

    string_value: str = betterproto.string_field(3, group="kind")
    """Represents a string value."""

    bool_value: bool = betterproto.bool_field(4, group="kind")
    """Represents a boolean value."""


@dataclass(eq=False, repr=False)
class Expression(betterproto.Message):
    """Provides a query expression type"""

    operand: str = betterproto.string_field(1)
    """The query operand or attribute"""

    operator: str = betterproto.string_field(2)
    """The query operator [ == | < | <= | > | >= | startsWith ]"""

    value: "ExpressionValue" = betterproto.message_field(3)
    """The query expression value"""


@dataclass(eq=False, repr=False)
class DocumentGetRequest(betterproto.Message):
    key: "Key" = betterproto.message_field(1)
    """Key of the document to retrieve"""


@dataclass(eq=False, repr=False)
class DocumentGetResponse(betterproto.Message):
    document: "Document" = betterproto.message_field(1)
    """The retrieved value"""


@dataclass(eq=False, repr=False)
class DocumentSetRequest(betterproto.Message):
    key: "Key" = betterproto.message_field(1)
    """Key of the document to set"""

    content: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(3)
    """The document content to store (JSON object)"""


@dataclass(eq=False, repr=False)
class DocumentSetResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DocumentDeleteRequest(betterproto.Message):
    key: "Key" = betterproto.message_field(1)
    """Key of the document to delete"""


@dataclass(eq=False, repr=False)
class DocumentDeleteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DocumentQueryRequest(betterproto.Message):
    collection: "Collection" = betterproto.message_field(1)
    """The collection to query"""

    expressions: List["Expression"] = betterproto.message_field(3)
    """Optional query expressions"""

    limit: int = betterproto.int32_field(4)
    """Optional query fetch limit"""

    paging_token: Dict[str, str] = betterproto.map_field(
        5, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """Optional query paging continuation token"""


@dataclass(eq=False, repr=False)
class DocumentQueryResponse(betterproto.Message):
    documents: List["Document"] = betterproto.message_field(1)
    """The retrieved values"""

    paging_token: Dict[str, str] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    """
    The query paging continuation token, when empty no further results are
    available
    """


@dataclass(eq=False, repr=False)
class DocumentQueryStreamRequest(betterproto.Message):
    collection: "Collection" = betterproto.message_field(1)
    """The collection to query"""

    expressions: List["Expression"] = betterproto.message_field(3)
    """Optional query expressions"""

    limit: int = betterproto.int32_field(4)
    """Optional query fetch limit"""


@dataclass(eq=False, repr=False)
class DocumentQueryStreamResponse(betterproto.Message):
    document: "Document" = betterproto.message_field(1)
    """The stream document"""


class DocumentServiceStub(betterproto.ServiceStub):
    async def get(
        self,
        document_get_request: "DocumentGetRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DocumentGetResponse":
        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Get",
            document_get_request,
            DocumentGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def set(
        self,
        document_set_request: "DocumentSetRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DocumentSetResponse":
        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Set",
            document_set_request,
            DocumentSetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def delete(
        self,
        document_delete_request: "DocumentDeleteRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DocumentDeleteResponse":
        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Delete",
            document_delete_request,
            DocumentDeleteResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query(
        self,
        document_query_request: "DocumentQueryRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "DocumentQueryResponse":
        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Query",
            document_query_request,
            DocumentQueryResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query_stream(
        self,
        document_query_stream_request: "DocumentQueryStreamRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["DocumentQueryStreamResponse"]:
        async for response in self._unary_stream(
            "/nitric.document.v1.DocumentService/QueryStream",
            document_query_stream_request,
            DocumentQueryStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response


class DocumentServiceBase(ServiceBase):
    async def get(
        self, document_get_request: "DocumentGetRequest"
    ) -> "DocumentGetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set(
        self, document_set_request: "DocumentSetRequest"
    ) -> "DocumentSetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(
        self, document_delete_request: "DocumentDeleteRequest"
    ) -> "DocumentDeleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query(
        self, document_query_request: "DocumentQueryRequest"
    ) -> "DocumentQueryResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_stream(
        self, document_query_stream_request: "DocumentQueryStreamRequest"
    ) -> AsyncIterator["DocumentQueryStreamResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get(
        self, stream: "grpclib.server.Stream[DocumentGetRequest, DocumentGetResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get(request)
        await stream.send_message(response)

    async def __rpc_set(
        self, stream: "grpclib.server.Stream[DocumentSetRequest, DocumentSetResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.set(request)
        await stream.send_message(response)

    async def __rpc_delete(
        self,
        stream: "grpclib.server.Stream[DocumentDeleteRequest, DocumentDeleteResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.delete(request)
        await stream.send_message(response)

    async def __rpc_query(
        self,
        stream: "grpclib.server.Stream[DocumentQueryRequest, DocumentQueryResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.query(request)
        await stream.send_message(response)

    async def __rpc_query_stream(
        self,
        stream: "grpclib.server.Stream[DocumentQueryStreamRequest, DocumentQueryStreamResponse]",
    ) -> None:
        request = await stream.recv_message()
        await self._call_rpc_handler_server_stream(
            self.query_stream,
            stream,
            request,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.document.v1.DocumentService/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentGetRequest,
                DocumentGetResponse,
            ),
            "/nitric.document.v1.DocumentService/Set": grpclib.const.Handler(
                self.__rpc_set,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentSetRequest,
                DocumentSetResponse,
            ),
            "/nitric.document.v1.DocumentService/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentDeleteRequest,
                DocumentDeleteResponse,
            ),
            "/nitric.document.v1.DocumentService/Query": grpclib.const.Handler(
                self.__rpc_query,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentQueryRequest,
                DocumentQueryResponse,
            ),
            "/nitric.document.v1.DocumentService/QueryStream": grpclib.const.Handler(
                self.__rpc_query_stream,
                grpclib.const.Cardinality.UNARY_STREAM,
                DocumentQueryStreamRequest,
                DocumentQueryStreamResponse,
            ),
        }
